import 'package:analyzer/dart/analysis/results.dart';
import 'package:analyzer/dart/ast/ast.dart';
import 'package:analyzer/dart/ast/visitor.dart';
import 'package:analyzer/src/ignore_comments/ignore_info.dart';

class ForbiddenMethodVisitor extends RecursiveAstVisitor<void> {
  final _nodes = <AstNode>[];
  final ResolvedUnitResult analysisResult;
  final String methodName;
  final String errorCode;

  ForbiddenMethodVisitor({this.analysisResult, this.methodName, this.errorCode});

  Iterable<AstNode> get nodes => _nodes;

  @override
  void visitMethodInvocation(MethodInvocation node) {
    node.visitChildren(this);
    if (node.methodName.name.contains(methodName)) {
      int lineNumber = analysisResult.unit.lineInfo.getLocation(node.offset).lineNumber;
      var ignoreInfo = IgnoreInfo.forDart(analysisResult.unit, analysisResult.content);
      if (!ignoreInfo.ignoredAt(errorCode, lineNumber)) {
        _nodes.add(node);
      }
    }
  }
}

class ForbiddenMethodVisitor2 extends RecursiveAstVisitor<void> {
  final _nodes = <AstNode>[];
  final ParsedUnitResult analysisResult;
  final String methodName;
  final String errorCode;

  ForbiddenMethodVisitor2({this.analysisResult, this.methodName, this.errorCode});

  Iterable<AstNode> get nodes => _nodes;

  @override
  void visitMethodInvocation(MethodInvocation node) {
    node.visitChildren(this);
    if (node.methodName.name.contains(methodName)) {
      int lineNumber = analysisResult.unit.lineInfo.getLocation(node.offset).lineNumber;
      var ignoreInfo = IgnoreInfo.forDart(analysisResult.unit, analysisResult.content);
      if (!ignoreInfo.ignoredAt(errorCode, lineNumber)) {
        _nodes.add(node);
      }
    }
  }
}
